%{
/* cfgtoken.l -- Tokenize the configuration file.
   $Id: cfgtoken.l,v 0.1 1998/07/04 17:03:06 eagle Exp $

   Written by Russ Allbery <rra@stanford.edu>
   Copyright 1998 by the Board of Trustees, Leland Stanford Jr. University

   Tokenize the configuration file.  This is a fairly smart lexer, since we
   want to give it sufficient intelligence to figure out when to possibly
   expect a number and do the conversion here.  (That saves us lots of
   trouble at the yacc stage of things.)  Note that we're not *quite*
   intelligent enough to be able to handle all-numeric log names, but we do
   allow quoted strings to coerce matters. */

#include "cfgparse.h"           /* Token defines and yacc interface */
#include "utility.h"            /* xmalloc() */

#include <string.h>             /* strcpy(), strncpy() */

/* cfgdie() is defined in cfgparse.y.  Lexers and parsers are incestuous. */
extern void cfgdie (char *format, ...);
  
/* The current line number of the file.  We keep track of this ourselves
   since although lex maintains yylineno, flex doesn't by default, and it's
   easy for us. */
int cfgline = 1;
%}

%start  args block command set value

LETTER  [a-zA-Z]
DIGIT   [0-9]
NAME    {LETTER}({LETTER}|{DIGIT}|_)*
WORD    [^ \t\n]+
NUMBER  {DIGIT}+
STRING  \"[^\"]*\"

%%

[ \t]+                  /* Ignore whitespace except for newlines. */

<INITIAL,block>#.*\n    cfgline++;

<INITIAL>set            BEGIN set; return SET;

<set>{NAME}             {
                          yylval.string = (char *) xmalloc (yyleng + 1);
                          strcpy (yylval.string, yytext);
                          return VARIABLE;
                        }

<set>=                  BEGIN value; return EQ;

<INITIAL>{NAME}         {
                          BEGIN block;
                          yylval.string = (char *) xmalloc (yyleng + 1);
                          strcpy (yylval.string, yytext);
                          return LABEL;
                        }

<block>\{               return LBRACE;
<block>\}               BEGIN INITIAL; return RBRACE;

<block>archive[ \t]*:   BEGIN args; return ARCHIVE;
<block>copy[ \t]*:      BEGIN args; return COPY;
<block>log[ \t]*:       BEGIN args; return LOG;
<block>restart[ \t]*:   BEGIN command; return RESTART;
<block>stop[ \t]*:      BEGIN command; return STOP;

<command>run            BEGIN args; return RUN;
<command>hup            BEGIN args; return HUP;

<args>{NUMBER}          {
                          yylval.number = atoi (yytext);
                          return NUMBER;
                        }

<args,value>{WORD}      {
                          yylval.string = (char *) xmalloc (yyleng + 1);
                          strcpy (yylval.string, yytext);
                          return STRING;
                        }

<args,value>{STRING}    {
                          yylval.string = (char *) xmalloc (yyleng - 1);
                          strncpy (yylval.string, yytext + 1, yyleng - 2);
                          yylval.string[yyleng - 2] = '\0';
                          return STRING;
                        }

<args>\n                BEGIN block; cfgline++; return EOL;
<value>\n               BEGIN INITIAL; cfgline++; return EOL;
<INITIAL,block>\n       cfgline++;

\n                      cfgdie ("Unexpected end of line");
.                       cfgdie ("Syntax error at or near `%c'", yytext[0]);

%%

/* Ensure that we exit at the end of the file. */
int yywrap () { return 1; }
